@@ -242,7 +242,6 @@ class ApplicationPresenter(MVP.BasicPresenter):
         self._status_presenter = StatusPresenter(view = widget_tree.get_widget("main_statusbar"))
         self._menufp_presenter = MenuFeedPropsPresenter( view = widget_tree.get_widget('menu_feed_properties'))
         self._menufp_presenter.set_sensitive(False)
-        self._offline_presenter.show()
         self._feedinfo_presenter = FeedInfoPresenter(view = FeedInfoView(widget_tree.get_widget('article_refresh_hbox')))
         self._find_presenter = FindPresenter(view=FindView(widget_tree.get_widget("find_vbox")))
         return
@@ -690,14 +689,24 @@ class ApplicationView(MVP.WidgetView):
         self._presenter.mark_all_as_read()
 
     def _on_find_activate(self, widget, *args):
+        xml = gtk.glade.get_widget_tree(self._widget)
+        menu_find = xml.get_widget('menu_find')
+        accel_label = menu_find.get_child()
+        
         if not self._find_toggled:
             self._presenter.show_search()
             self._detach_unmodified_accelerator_group()
             self._find_toggled = True
+            
+            # save the "Find..." stock text for later recovery
+            self._old_label_text = accel_label.get_text()
+            accel_label.set_text(_('Return to feed list...'))
+            
         else:
             self._presenter.hide_search()
             self._attach_unmodified_accelerator_group()
             self._find_toggled = False
+            accel_label.set_text(self._old_label_text)            
 
     def _on_menu_next_activate(self, *args):
         self._presenter.display_next_item()
@@ -769,6 +778,7 @@ import os
 import gettext
 import getopt
 import sys
+import strawdbus
 
 class Application:
     def __init__(self):
@@ -817,6 +827,7 @@ class Application:
         iconfile = os.path.join(utils.find_image_dir(),"straw.png")
         gtk.window_set_default_icon(gtk.gdk.pixbuf_new_from_file(iconfile))
 
+        strawdbus.start_services()
 
     def mainloop(self):
         threads = Config.get_instance().use_threads
@@ -857,18 +868,6 @@ class Application:
         locale.setlocale(locale.LC_ALL, '')
         return
 
-    # fix this as optional modules get bigger
-    def load_dbus(self):
-        try:
-            import dbus
-        except ImportError:
-            pass
-        else:
-            #Init dbus connection
-            from strawdbus import FeedReader
-            feedreader = FeedReader()
-        return
-
     def load_tray(self):
         from Tray import Tray
         tray = Tray()
