@@ -1,3 +1,23 @@
+""" Config.py
+
+Module for Straw's configuration settings.
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """ GNU General Public License
+
+This program is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
 import cPickle, os, traceback
 from urlparse import urlparse
 from error import log, logtb, logparam
@@ -12,15 +32,17 @@ GCONF_STRAW_ROOT = "/apps/straw"
 OPTION_LAST_POLL = "/general/last_poll"
 OPTION_ITEMS_STORED = "/general/number_of_items_stored"
 OPTION_ITEM_ORDER = "/general/item_order_newest"
+OPTION_BROWSER_CMD = "/general/browser_cmd"
 OPTION_WINDOW_SIZE_W = "/ui/window_width"
 OPTION_WINDOW_SIZE_H = "/ui/window_height"
 OPTION_MAIN_PANE_POS = "/ui/main_pane_position"
 OPTION_SUB_PANE_POS = "/ui/sub_pane_position"
+OPTION_WINDOW_MAX = "/ui/window_maximized"
+OPTION_MAGNIFICATION = "/ui/text_magnification"
 OPTION_OFFLINE = "/general/offline"
 OPTION_POLL_FREQUENCY = "/general/poll_frequency"
 OPTION_FEED_ID_SEQ = "feed_id_seq"
 OPTION_FEEDS = "feeds"
-OPTION_FEED_INFO_VISIBILITY = "/general/feed_info_visibility"
 OPTION_CATEGORIES = "categories"
 
 def ensure_directory(strawdir):
@@ -66,13 +88,15 @@ class ConfigGConfPersistence:
     SAVERS = {OPTION_LAST_POLL: 'int',
               OPTION_ITEMS_STORED: 'int',
               OPTION_ITEM_ORDER: 'bool',
+              OPTION_BROWSER_CMD: 'string',
               OPTION_WINDOW_SIZE_W: 'int',
               OPTION_WINDOW_SIZE_H: 'int',
               OPTION_MAIN_PANE_POS: 'int',
               OPTION_SUB_PANE_POS: 'int',
               OPTION_OFFLINE: 'bool',
-              OPTION_POLL_FREQUENCY: 'int',
-              OPTION_FEED_INFO_VISIBILITY: 'bool'}
+              OPTION_WINDOW_MAX: 'bool',
+              OPTION_MAGNIFICATION: 'float',
+              OPTION_POLL_FREQUENCY: 'int'}
 
     def __init__(self, client):
         self.client = client
@@ -104,44 +128,45 @@ class ConfigPersistence:
 class BaseProxyConfig(object):
     def __init__(self):
         self._config_working = False
-        
-    def get_config_working(self):
+
+    @property
+    def config_working(self):
         return self._config_working
 
-    config_working = property(get_config_working, None, None, "")
+    @apply
+    def host():
+        doc = ""
+        def fget(self):
+            return self._host
+        def fset(self, host):
+            self._host = host
+            self._ip = None
+        return property(**locals())
 
-    def get_host(self):
-        return self._host
+    @apply
+    def ip():
+        doc = ""
+        def fget(self):
+            return self._ip
+        def fset(self, ip):
+            assert self._ip is None, "Overwriting proxy IP not allowed!"
+            if ip is None:
+                self.use = 0
+            self._ip = ip
+        return property(**locals())
 
-    def set_host(self, host):
-        self._host = host
-        self._ip = None
+    @property
+    def is_waiting(self):
+        return self.use and not self._ip
+
+    def _ip_lookup_callback(self, host, ip, data):
+        self.ip = ip
 
     def lookup_host(self):
         if self._host and not self._ip:
             LookupManager.get_instance().lookup(
                 self._host, self._ip_lookup_callback)
 
-    host = property(get_host, set_host, None, "")
-
-    def get_ip(self):
-        return self._ip
-
-    def set_ip(self, ip):
-        assert self._ip is None, "Overwriting proxy IP not allowed!"
-        if ip is None:
-            self.use = 0
-        self._ip = ip
-
-    ip = property(get_ip, set_ip, None, "")
-
-    def _ip_lookup_callback(self, host, ip, data):
-        self.ip = ip
-
-    def _is_waiting(self):
-        return self.use and not self._ip
-
-    is_waiting = property(_is_waiting, None, None, "")
 
 class GconfProxyConfig(BaseProxyConfig):
     """Encapsulate proxy use and location information (host, port, ip),
@@ -279,206 +304,258 @@ class Config(object, Event.SignalEmitter):
         self.persistence = persistence
         self.initialize_slots(Event.ItemOrderChangedSignal,
                               Event.OfflineModeChangedSignal,
-                              Event.PollFrequencyChangedSignal)
+                              Event.PollFrequencyChangedSignal,
+                              Event.NumberOfItemsStoredChangedSignal)
+        self._feed_id_seq = 0
+        self._poll_freq = 1800
+        self._last_poll = 0
+        self._browser_cmd = ''
+        self._items_stored = 30
+        self._item_order = False
+        self._main_window_size = (640,480)
+        self._main_pane_position = 100
+        self._sub_pane_position = 100
+        self.first_time = None
+        self._offline = True
+        self._window_maximized = False
+        self._use_threads = True
+        self._reload_css = False
+        self._no_etags = False
+
+    def initialize_proxy(self):
         pcchoices = (GconfProxyConfig, EnvironmentProxyConfig, NullProxyConfig)
         for p in pcchoices:
             self._proxy_config = p()
             if self._proxy_config.config_working:
                 break
 
+    def initialize_options(self):
+        # Call this after calling Config's constructor
         self.first_time = ensure_directory(self._straw_dir)
 
         if os.path.exists(self.straw_config_file):
             self._feed_id_seq = self.persistence.load_option(OPTION_FEED_ID_SEQ)
-        else:
-            self._feed_id_seq = 0
 
         self._poll_freq = self.persistence.load_option(OPTION_POLL_FREQUENCY)
         self._last_poll = self.persistence.load_option(OPTION_LAST_POLL)
         self._items_stored = self.persistence.load_option(OPTION_ITEMS_STORED)
+        self._browser_cmd = self.persistence.load_option(OPTION_BROWSER_CMD)
         self._item_order = self.persistence.load_option(OPTION_ITEM_ORDER)
-        self._main_window_size = (
-            self.persistence.load_option(OPTION_WINDOW_SIZE_W),
-            self.persistence.load_option(OPTION_WINDOW_SIZE_H))
+
+        width = self.persistence.load_option(OPTION_WINDOW_SIZE_W)
+        height = self.persistence.load_option(OPTION_WINDOW_SIZE_H)
+        if width <= 0:
+            width = 640
+        if height <= 0:
+            height = 480
+        self._main_window_size = (width, height)
+
         self._main_pane_position = self.persistence.load_option(
             OPTION_MAIN_PANE_POS)
         self._sub_pane_position = self.persistence.load_option(
             OPTION_SUB_PANE_POS)
+        self._window_maximized = self.persistence.load_option(OPTION_WINDOW_MAX)
+        self._text_magnification = self.persistence.load_option(OPTION_MAGNIFICATION)
         self._offline = self.persistence.load_option(OPTION_OFFLINE)
-        self._feed_info_visibility = self.persistence.load_option(OPTION_FEED_INFO_VISIBILITY)
-        self._use_threads = True
-        self._reload_css = False
 
-    def get_straw_dir(self):
+    @property
+    def straw_dir(self):
         return self._straw_dir
 
-    straw_dir = property(get_straw_dir, None, None, "")
-
-    def get_straw_config_file(self):
+    @property
+    def straw_config_file(self):
         return self._straw_config_file
 
-    straw_config_file = property(get_straw_config_file, None, None, "")
-    
-    def get_proxy_config(self):
+    @property
+    def proxy_config(self):
         return self._proxy_config
 
-    proxy_config = property(get_proxy_config, None, None, "")
+    @apply
+    def feeds():
+        doc = "Marshalled feed data"
+        def fget(self):
+            return self.persistence.load_option(OPTION_FEEDS)
+        def fset(self, feeddata):
+            self.persistence.save_option(
+                OPTION_FEEDS, feeddata)
+        return property(**locals())
 
-    # and the rest of the stuff
-    def set_feeds(self, feeddata):
-        self.persistence.save_option(
-            OPTION_FEEDS, feeddata)
-        return
+    @apply
+    def categories():
+        doc = ""
+        def fget(self):
+            return self.persistence.load_option(OPTION_CATEGORIES)
+        def fset(self, categorydata):
+            self.persistence.save_option(
+                OPTION_CATEGORIES, categorydata)
+        return property(**locals())
+    
+    @apply
+    def poll_frequency():
+        doc = "Polling frequency"
+        def fget(self):
+            return self._poll_freq
+        def fset(self, poll_frequency):
+            if self._poll_freq != poll_frequency:
+                self._poll_freq = poll_frequency
+                self.persistence.save_option(OPTION_POLL_FREQUENCY, poll_frequency)
+                self.emit_signal(Event.PollFrequencyChangedSignal(self, poll_frequency))
+        return property(**locals())
 
-    def get_feeds(self):
-        return self.persistence.load_option(OPTION_FEEDS)
+    @apply
+    def last_poll():
+        doc = "last poll"
+        def fget(self):
+            return self._last_poll
+        def fset(self, last_poll):
+            if self._last_poll != last_poll:
+                self._last_poll = last_poll
+                self.persistence.save_option(OPTION_LAST_POLL, last_poll)
+        return property(**locals())
 
-    feeds = property(get_feeds, set_feeds, None, "Marshalled feed data")
+    @apply
+    def  browser_cmd():
+        doc = "The browser to use"
+        def fget(self):
+            return self._browser_cmd
+        def fset(self, browser_cmd):
+            if self._browser_cmd != browser_cmd:
+                self._browser_cmd = browser_cmd
+                self.persistence.save_option(OPTION_BROWSER_CMD, browser_cmd)
+        return property(**locals())
 
-    def set_categories(self, categorydata):
-        self.persistence.save_option(
-            OPTION_CATEGORIES, categorydata)
-        return
+    @apply
+    def number_of_items_stored():
+        doc = "Number of items to store per feed"
+        def fget(self):
+            return self._items_stored
+        def fset(self, num=30):
+            if self._items_stored != num:
+                self._items_stored = num
+                self.persistence.save_option(OPTION_ITEMS_STORED, num)
+                self.emit_signal(Event.NumberOfItemsStoredChangedSignal(self))
+        return property(**locals())
 
-    def get_categories(self):
-        return self.persistence.load_option(OPTION_CATEGORIES)
+    @apply
+    def item_order():
+        doc = "Ordering of Items"
+        def fget(self):
+            return self._item_order
+        def fset(self, order):
+            if self._item_order != order:
+                self._item_order = order
+                self.persistence.save_option(OPTION_ITEM_ORDER, order)
+                self.emit_signal(Event.ItemOrderChangedSignal(self))
+        return property(**locals())
 
-    categories = property(get_categories, set_categories)
-
-    def set_poll_frequency(self, poll_frequency):
-        if self._poll_freq != poll_frequency:
-            self._poll_freq = poll_frequency
-            self.persistence.save_option(OPTION_POLL_FREQUENCY, poll_frequency)
-            self.emit_signal(Event.PollFrequencyChangedSignal(self, poll_frequency))
-        return
-
-    def get_poll_frequency(self):
-        return self._poll_freq
-
-    poll_frequency = property(get_poll_frequency, set_poll_frequency, doc="Polling frequency")
-
-    def set_last_poll(self, last_poll):
-        if self._last_poll != last_poll:
-            self._last_poll = last_poll
-            self.persistence.save_option(OPTION_LAST_POLL, last_poll)
-        return
-
-    def get_last_poll(self):
-        return self._last_poll
-
-    last_poll = property(get_last_poll, set_last_poll, doc="Last polled")
-
-    def get_number_of_items_stored(self):
-        return self._items_stored
-
-    def set_number_of_items_stored(self, num=30):
-        if self._items_stored != num:
-            self._items_stored = num
-            self.persistence.save_option(OPTION_ITEMS_STORED, num)
-            self.emit_signal(Event.NumberOfItemsStoredChangedSignal(self))
-        return
-
-    number_of_items_stored = property(get_number_of_items_stored,
-                                      set_number_of_items_stored,
-                                      doc="Number of items to store per feed")
-
-    def set_item_order(self, order):
-        if self._item_order != order:
-            self._item_order = order
-            self.persistence.save_option(OPTION_ITEM_ORDER, order)
-            self.emit_signal(Event.ItemOrderChangedSignal(self))
-        return
-
-    def get_item_order(self):
-        return self._item_order
-
-    item_order = property(get_item_order, set_item_order, doc="Ordering of items")
-
-
-    def set_feed_info_visibility(self, visibility):
-        if self._feed_info_visibility != visibility:
-            self._feed_info_visibility = visibility
-            self.persistence.save_option(OPTION_FEED_INFO_VISIBILITY, visibility)
-        return
-
-    def get_feed_info_visibility(self):
-        return self._feed_info_visibility
-
-    feed_info_visibility = property(get_feed_info_visibility,
-                                 set_feed_info_visibility, doc="Sets the visibility of feed info widget")
-
-    def get_feed_id_seq(self):
-        return self._feed_id_seq
-
-    def set_feed_id_seq(self, id):
-        self._feed_id_seq = id
-        self.persistence.save_option(OPTION_FEED_ID_SEQ, id)
-        return
+    @apply
+    def feed_id_seq():
+        doc = ""
+        def fget(self):
+            return self._feed_id_seq
+        def fset(self, id):
+            self._feed_id_seq = id
+            self.persistence.save_option(OPTION_FEED_ID_SEQ, id)
+        return property(**locals())
 
     def next_feed_id_seq(self):
         self.feed_id_seq += 1
         return self._feed_id_seq
 
-    feed_id_seq = property(get_feed_id_seq, set_feed_id_seq, None, "")
+    @apply
+    def main_window_size():
+        doc = ""
+        def fget(self):
+            return self._main_window_size
+        def fset(self, size):
+            if self._main_window_size != size:
+                self._main_window_size = size
+                self.persistence.save_option(OPTION_WINDOW_SIZE_W, size[0])
+                self.persistence.save_option(OPTION_WINDOW_SIZE_H, size[1])
+        return property(**locals())
 
-    def set_main_window_size(self, size):
-        if self._main_window_size != size:
-            self._main_window_size = size
-            self.persistence.save_option(OPTION_WINDOW_SIZE_W, size[0])
-            self.persistence.save_option(OPTION_WINDOW_SIZE_H, size[1])
-            self.emit_signal(Event.MainWindowSizeChangedSignal(self))
-        return
+    @apply
+    def offline():
+        doc = ""
+        def fget(self):
+            return self._offline
+        def fset(self, mode):
+            if self._offline != mode:
+                self._offline = mode
+                self.persistence.save_option(OPTION_OFFLINE, mode)
+                self.emit_signal(Event.OfflineModeChangedSignal(self))
+        return property(**locals())
 
-    def get_main_window_size(self):
-        return self._main_window_size
+    @apply
+    def window_maximized():
+        doc = ""
+        def fget(self):
+            return self._window_maximized
+        def fset(self, state):
+            if self._window_maximized is not state:
+                self._window_maximized = state
+                self.persistence.save_option(OPTION_WINDOW_MAX, state)
+        return property(**locals())
 
-    main_window_size = property(get_main_window_size, set_main_window_size, "")
+    @apply
+    def main_pane_position():
+        doc = ""
+        def fget(self):
+            return self._main_pane_position
+        def fset(self, position):
+            if self._main_pane_position != position:
+                self._main_pane_position = position
+                self.persistence.save_option(OPTION_MAIN_PANE_POS, position)
+        return property(**locals())
 
-    def set_offline(self, mode):
-        if self._offline != mode:
-            self._offline = mode
-            self.persistence.save_option(OPTION_OFFLINE, mode)
-            self.emit_signal(Event.OfflineModeChangedSignal(self))
-        return
+    @apply
+    def sub_pane_position():
+        doc = ""
+        def fget(self):
+            return self._sub_pane_position
+        def fset(self, position):
+            if self._sub_pane_position != position:
+                self._sub_pane_position = position
+                self.persistence.save_option(OPTION_SUB_PANE_POS, position)
+        return property(**locals())
 
-    def get_offline(self):
-        return self._offline
+    @apply
+    def text_magnification():
+        doc = "sets the amount of magnification in the item view"
+        def fget(self):
+            return self._text_magnification
+        def fset(self, tm):
+            if self._text_magnification is not tm:
+                self._text_magnification = tm
+                self.persistence.save_option(OPTION_MAGNIFICATION, tm)
+        return property(**locals())
 
-    offline = property(get_offline, set_offline, None, "")
+    @apply
+    def use_threads():
+        doc = ""
+        def fget(self):
+            return self._use_threads
+        def fset(self, threads):
+            self._use_threads = threads
+        return property(**locals())
 
-    def set_main_pane_position(self, position):
-        if self._main_pane_position != position:
-            self._main_pane_position = position
-            self.persistence.save_option(OPTION_MAIN_PANE_POS, position)
-            self.emit_signal(Event.MainPanePositionChangedSignal(self))
+    @apply
+    def reload_css():
+        doc = ""
+        def fget(self):
+            return self._reload_css
+        def fset(self, reload_css):
+            self._reload_css = reload_css
+        return property(**locals())
 
-    def get_main_pane_position(self):
-        return self._main_pane_position
-
-    main_pane_position = property(get_main_pane_position, set_main_pane_position, None, "")
-
-    def set_sub_pane_position(self, position):
-        if self._sub_pane_position != position:
-            self._sub_pane_position = position
-            self.persistence.save_option(OPTION_SUB_PANE_POS, position)
-            self.emit_signal(Event.SubPanePositionChangedSignal(self))
-
-    def get_sub_pane_position(self):
-        return self._sub_pane_position
-
-    sub_pane_position = property(get_sub_pane_position, set_sub_pane_position, None, "")
-
-    def set_use_threads(self, threads):
-        self._use_threads = threads
-
-    def get_use_threads(self):
-        return self._use_threads
-
-    use_threads = property(get_use_threads, set_use_threads)
-
-    def set_reload_css(self, reload): self._reload_css = reload
-    def get_reload_css(self): return self._reload_css
-    reload_css = property(get_reload_css, set_reload_css)
+    @apply
+    def no_etags():
+        doc = ""
+        def fget(self):
+            return self._no_etags
+        def fset(self, no_etags):
+            self._no_etags = no_etags
+        return property(**locals())
 
 def convert_if_necessary(config):
     if not os.path.exists(config.straw_config_file):
@@ -489,17 +566,16 @@ def convert_if_necessary(config):
         cf = cPickle.load(f)
 
         feeds = cf.get('feeds', None)
-        if feeds is not None and feeds[0].get('_n_items_unread', None) is None:
-            import ItemStore
+        if feeds and feeds[0].get('_n_items_unread', None) is None:
 
             def _convert_feeds(feeds, parent):
+                import ItemStore
                 itemstore = ItemStore.get_instance(config.straw_dir)
-
                 for feed in feeds:
                     if isinstance(feed, list):
                         _convert_feeds(feed[1:], parent)
                     else:
-                        stored = feed['_items_stored']
+                        stored = feed.get('_items_stored', -1)
                         if stored > -1:
                             cutoff = stored
                         else:
@@ -547,13 +623,16 @@ def create_instance():
     # Add your GConf key here as well!
     cp = ConfigPersistence(
         (create_gconf_persistence(),
-         (OPTION_LAST_POLL, OPTION_ITEMS_STORED, OPTION_ITEM_ORDER,
+         (OPTION_LAST_POLL, OPTION_ITEMS_STORED, OPTION_ITEM_ORDER, OPTION_BROWSER_CMD,
           OPTION_WINDOW_SIZE_W, OPTION_WINDOW_SIZE_H,
           OPTION_MAIN_PANE_POS, OPTION_SUB_PANE_POS, OPTION_OFFLINE,
-          OPTION_POLL_FREQUENCY, OPTION_FEED_INFO_VISIBILITY)),
+          OPTION_MAGNIFICATION,
+          OPTION_POLL_FREQUENCY, OPTION_WINDOW_MAX)),
         (create_pickle_persistence(),
          (OPTION_FEED_ID_SEQ, OPTION_FEEDS, OPTION_CATEGORIES)))
     config = Config(cp)
+    config.initialize_proxy()
+    config.initialize_options()
     convert_if_necessary(config)
     return config
 
