@@ -37,35 +37,46 @@ class CacheEntry(object):
         if not restore:
             self._save_count()
 
-    def get_image(self): return self._image
-    def set_image(self, image): self._image = image
-    image = property(get_image, set_image)
+    @apply
+    def image():
+        doc="The image object that is associated with this class"
+        def fget(self):
+            return self._image
+        def fset(self, image):
+            self._image = image
+        return property(**locals())
 
-    def get_count(self): return self._count
+    @apply
+    def count():
+        doc="The number of instances of this image"
+        def fget(self):
+            return self._count
+        def fset(self, count):
+            self._count = count
+            self._save_count()
+        return property(**locals())
 
-    def set_count(self, count):
-        self._count = count
-        self._save_count()
-        
-    count = property(get_count, set_count)
+    @apply
+    def pollstopper():
+        doc="the stopper object for this cache entry"
+        def fget(self):
+            return self._pollstopper
+        def fset(self, pollstopper):
+            self._pollstopper = pollstopper
+            if pollstopper:
+                pollstopper.signal_connect(Event.PollingStoppedSignal,
+                                           self._polling_stopped)
+        return property(**locals())
 
     def incr_count(self): self._count += 1
     def decr_count(self): self._count -= 1
 
-    def get_pollstopper(self): return self._pollstopper
-    def set_pollstopper(self, pollstopper):
-        self._pollstopper = pollstopper
-        if pollstopper is not None:
-            pollstopper.signal_connect(Event.PollingStoppedSignal,
-                                       self._polling_stopped)
-    pollstopper = property(get_pollstopper, set_pollstopper)
-
     def _polling_stopped(self):
         self._pollstopper = None
 
     def _save_count(self):
         ItemStore.get_instance().set_image_count(self._image.url, self._count)
-        
+
 
 class Cache(Event.SignalEmitter):
     """Image cache with explicit reference counting."""
@@ -173,7 +184,7 @@ cache = Cache()
 class ImageConsumer:
     def __init__(self, imobj):
         self._imobj = imobj
-    
+
     def http_results(self, status, headers, data):
         if status[1] == 200:
             self._imobj.set_data(data)
@@ -194,5 +205,3 @@ def initialize():
         libdir + "/image-waiting.png").read()
     STATUS_IMAGE_BROKEN = open(
         libdir + "/image-broken.png").read()
-    for uri, count in ItemStore.get_instance().get_image_counts():
-        cache.set_image_with_count(uri, Image(uri), count, restore=True)
