@@ -60,6 +60,8 @@ class MyDB:
         self._env.set_data_dir(dbhome)
         self._env.set_lk_detect(DB_LOCK_DEFAULT)  # enable auto deadlock avoidance
         self._env.set_lg_max(2**20)
+        self._env.set_lk_max_locks   (10000)
+        self._env.set_lk_max_objects (10000)
 
         try:
             self._env.open(dbhome, recoverenv | flagsforenv, mode)
@@ -119,32 +121,8 @@ class MyDB:
     def begin_transaction(self):
         return self._env.txn_begin()
 
-    def get_feed_item_ids(self, feed, txn):
-        key = "fids:%d" % feed.id
-        ids = self._get_item_ids(key, txn)
-        return ids
-        #log("TEMPORARY HACK: filtering out duplicate ids from feed item id list. This should be disabled (and unnecessary) before release.")
-        #seen = {}
-        #realids = []
-        #for itemid in ids:
-        #    seen[itemid] = seen.get(itemid, 0) + 1
-        #    if seen[itemid] > 1:
-        #        log("Found duplicate id ", itemid, ", filtering")
-        #        continue
-        #    realids.append(itemid)
-        #if len(realids) < len(ids):
-        #    log("Saving the new feed id list")
-        #    self.save_feed_item_ids(feed, realids, txn)
-        #return realids
-
-    def get_item_ids_for_id(self, iid, txn):
-        # Used for low-level stuff
-        # Hopefully, this will be used in the future for more separation
-        # Currently used by config conversion
+    def get_item_ids(self, iid, txn):
         key = "fids:%d" % iid
-        return self._get_item_ids(key, txn)
-
-    def _get_item_ids(self, key, txn):
         dids = self._db.get(key, txn=txn)
         ids = []
         if dids:
@@ -171,34 +149,14 @@ class MyDB:
             if commit:
                 txn.commit()
 
-    def get_item(self, feed, item_id, txn=None):
-        return self._get_item(feed.id, item_id, txn)
-
-    def get_item_for_id(self, iid, item_id, txn=None):
-        return self._get_item(iid, item_id, txn)
-
-    def _get_item(self, iid, item_id, txn=None):
-        item = self._db.get("%d:%d" % (iid, item_id), txn=txn)
+    def get_item(self, feed_id, item_id, txn=None):
+        item = self._db.get("%d:%d" % (feed_id, item_id), txn=txn)
         return unstringify_item(item)
 
-    def add_item(self, item):
-        txn = self.begin_transaction()
-        try:
-            feed_item_ids = self.get_feed_item_ids(item.feed, txn=txn)
-            feed_item_ids.append(item.id)
-            self.save_feed_item_ids(feed, feed_ids, txn)
-            self._db.put("%d:%d" % (item.feed.id, item.id),
-                         stringify_item(item), txn=txn)
-        except Exception, ex:
-            txn.abort()
-            logtb(str(ex))
-        else:
-            txn.commit()
-
     def add_items(self, feed, items):
         txn = self.begin_transaction()
         try:
-            feed_item_ids = self.get_feed_item_ids(feed, txn=txn)
+            feed_item_ids = self.get_item_ids(feed.id, txn=txn)
             for item in items:
                 self._db.put("%d:%d" % (item.feed.id, item.id), stringify_item(item), txn=txn)
                 # TODO: it might be a good idea to check here that we don't add
@@ -213,11 +171,6 @@ class MyDB:
         else:
             txn.commit()
 
-    def del_item(self, feed, items):
-        log("DEPRECATED: use delete_items")
-        self.delete_items(feed, items)
-
-
     def delete_items(self, feed, items):
         """ Deletes a list of items.
 
@@ -225,7 +178,7 @@ class MyDB:
         """
         txn = self.begin_transaction()
         try:
-            feed_item_ids = self.get_feed_item_ids(feed, txn=txn)
+            feed_item_ids = self.get_item_ids(feed.id, txn=txn)
             # because of bugs, we sometime get here duplicate ids. instead of dying,
             # warn the user but continue
             item_ids = []
@@ -264,9 +217,9 @@ class MyDB:
         txn = self.begin_transaction()
         items = []
         try:
-            ids = self.get_feed_item_ids(feed, txn=txn)
+            ids = self.get_item_ids(feed.id, txn=txn)
             for id in ids:
-                item = self.get_item(feed, id, txn=txn)
+                item = self.get_item(feed.id, id, txn=txn)
                 if item is not None:
                     items.append(item)
         except Exception, ex:
@@ -285,9 +238,9 @@ class MyDB:
         txn = self.begin_transaction()
         num_unread = 0
         try:
-            ids = self.get_item_ids_for_id(fid, txn=txn)
+            ids = self.get_item_ids(fid, txn=txn)
             for id in ids[len(ids)-cutoff:]:
-                item = self.get_item_for_id(fid, id, txn=txn)
+                item = self.get_item(fid, id, txn=txn)
                 if item is not None and item.seen == 0:
                     num_unread += 1
                 else: continue
@@ -491,13 +444,12 @@ class MyDB:
                         break
                 for image, count in images.items():
                     key = ("imagecount:" + image).encode('utf-8')
-                    # TODO: remove debugging message
-                    log("storing count ", count, " for key ", key)
                     cur.put(key, str(count), DB_KEYFIRST)
                 imagelistcursor.put("images", pickle.dumps(images.keys()), DB_CURRENT)
             finally:
                 cur.close()
-                imagelistcursor.close()
+                if imagelistcursor != None:
+                    imagelistcursor.close()
         except Exception, ex:
             txn.abort()
             raise
@@ -564,9 +516,6 @@ class ItemStore:
         self._stop = False
         self._action_queue = []
 
-    #def initialize_db(self):
-    #    self._db.CreateTable("items", ["feed_id", "item_id", "item_data"])
-
     def _feed_created_cb(self, signal):
         self._connect_feed_signals(signal.feed)
 
@@ -699,7 +648,8 @@ def stringify_item(item):
         'publication_number': item.publication_number,
         'publication_section': item.publication_section,
         'publication_starting_page': item.publication_starting_page,
-        'sticky': item._sticky}
+        'sticky': item._sticky,
+        'enclosures': item.enclosures}
     return pickle.dumps(itemdict)
 
 def unstringify_item(itemstring):
@@ -728,6 +678,7 @@ def unstringify_item(itemstring):
     item.contributors = idict.get('contributors', None)
     item.license_urls = idict.get('license_urls', None)
     item._sticky = idict.get('sticky', 0)
+    item.enclosures = idict.get('enclosures', None)
     item.publication_name = idict.get('publication_name', None)
     item.publication_volume = idict.get('publication_volume', None)
     item.publication_number = idict.get('publication_number', None)
