@@ -411,6 +411,44 @@ class HTMLPresenter(MVP.BasicPresenter):
         ret.append('<p></p>')
         # Additional information
         dcret = []
+        
+        # RSS Enclosures
+        
+        if item.enclosures:
+            dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><table>' % _("Enclosed Media"))
+            for enc in item.enclosures: # rss 2.0 defines only one enclosure per item
+                size = int(enc.length)
+                unit = _('bytes')
+                if size > 1024:
+                    unit = _('KB')
+                    size /= 1024.0
+                if size > 1024:
+                    unit = _('MB')
+                    size /= 1024.0
+                link_text = enc['href'].split('/')[-1]
+                
+                # find what kind of media is that. enc[type] will have something like audio/mp3 or video/mpeg (mimetypes)
+                # some symlinks are not present on the tango icon theme mimetype dir. audio and application are 2 good examples. So I am not relying on the symlinks now...
+                kind = enc['type'].split('/')[0]
+                if kind == 'audio':
+                    icon_name = 'audio-x-generic'
+                elif kind == 'video':
+                    icon_name = 'video-x-generic'
+                elif kind == 'image':
+                    icon_name = 'image-x-generic'
+                elif kind == 'application':
+                    icon_name = 'binary'
+                elif kind == 'text':
+                    icon_name = 'text-x-generic'
+                else:
+                    icon_name = "unknown"
+                
+                it = gtk.icon_theme_get_default()
+                ii = it.lookup_icon(icon_name, 32, gtk.ICON_LOOKUP_NO_SVG)
+                imgsrc = 'file://' + ii.get_filename()
+                dcret.append('<tr><td><div style="vertical-align: middle"><a class="dclink" href="%s" style="vertical-align: middle"><img style="padding: 0px 15px 5px 0px" src=%s /> %s</a> (%.2f %s - %s)</div></td></tr>' % (enc['href'], imgsrc, link_text, size, unit, enc['type']))
+            dcret.append('</table></td></tr>')
+        
         if item.creator is not None:
             dcret.append('<tr class="tr.dc"><td class="dcname"><span>%s</span></td><td class="dcvalue"><span>%s</span></td></tr>' % (_("Posted by"), item.creator))
         if item.contributors is not None and len(item.contributors):
