@@ -1,11 +1,32 @@
+""" MVP.py
+
+Module that provides a Model-View-Presenter framework for use with Straw
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """
+Straw is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+Straw is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
+
 import pygtk
 pygtk.require('2.0')
 from gtk import glade
-import straw
+import Event
+
 
 __doc__ = "A MVP framework. Contains base classes for views and presenters."
 
-class WidgetView(object):
+class View(object):
     """Base class for widget-based view objects."""
     def __init__(self, widget, model = None, presenter = None):
         self._widget = widget
@@ -18,13 +39,10 @@ class WidgetView(object):
             self._presenter_set()
 
         self._initialize()
+        self._connect_signals(widget)
 
-        xml = glade.get_widget_tree(widget)
-        nameFuncMap = {}
-        for key in dir(self.__class__):
-            if key[:4] == '_on_':
-                nameFuncMap[key[1:]] = getattr(self, key)
-        xml.signal_autoconnect(nameFuncMap)
+    def _connect_signals(self, widget):
+        raise NotImplemented
 
     def _initialize(self):
         """Called from the constructor after the instance variables have
@@ -45,8 +63,12 @@ class WidgetView(object):
 
     def set_model(self, model):
         self._model = model
-        self._model_set()
-    model = property(get_model, set_model, None, "The model object.")
+        if model:
+            # don't call model_set if model is None. This prevents objects
+            # relying on model_set to fail (or gtk throwing a warning when
+            # model is not a gtk.TreeModel).
+            self._model_set()
+    model = property(get_model, set_model, None, doc="The model object.")
 
     def get_presenter(self):
         return self._presenter
@@ -57,10 +79,33 @@ class WidgetView(object):
     presenter = property(get_presenter, set_presenter, None,
                          "The presenter object.")
 
-class BasicPresenter(object, straw.SignalEmitter):
+class WidgetView(View):
+    def __init__(self, widget, model=None, presenter=None):
+        View.__init__(self, widget, model, presenter)
+
+    def _connect_signals(self, widget):
+        xml = glade.get_widget_tree(widget)
+        nameFuncMap = {}
+        for key in dir(self.__class__):
+            if key[:4] == '_on_':
+                nameFuncMap[key[1:]] = getattr(self, key)
+        xml.signal_autoconnect(nameFuncMap)
+
+class GladeView(View):
+    def __init__(self, widget, model=None, presenter=None):
+        View.__init__(self, widget, model, presenter)
+
+    def _connect_signals(self, widget):
+        nameFuncMap = {}
+        for key in dir(self.__class__):
+            if key[:4] == '_on_':
+                nameFuncMap[key[1:]] = getattr(self, key)
+        widget.signal_autoconnect(nameFuncMap)
+
+class BasicPresenter(object, Event.SignalEmitter):
     """Base class for basic (non-composite) presenters."""
     def __init__(self, model = None, view = None):
-        straw.SignalEmitter.__init__(self)
+        Event.SignalEmitter.__init__(self)
 
         self._model = model
         self._view = view
