@@ -1,14 +1,38 @@
+""" PreferencesDialog.py
+
+Module setting user preferences.
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """ GNU General Public License
+
+Straw is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+Straw is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
 import pygtk
 pygtk.require('2.0')
 import gobject
 import gtk
 import pango
 from gtk import glade
-import straw
 import locale
 import error
 import MVP
 from weakref import WeakKeyDictionary
+import Event
+import FeedCategoryList
+import FeedList
+import Config
+import ValueMonitor
 
 class CategoryListView(MVP.WidgetView):
     """View object for the category tree view"""
@@ -20,29 +44,54 @@ class CategoryListView(MVP.WidgetView):
     def _initialize(self):
         treemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
                                   gobject.TYPE_INT, gobject.TYPE_BOOLEAN)
+        treemodel.set_sort_func(self.COLUMN_TITLE, self._sort_feeds)
+        treemodel.set_sort_column_id(self.COLUMN_TITLE, gtk.SORT_ASCENDING)
         self._widget.set_model(treemodel)
         self._widget.set_rules_hint(False)
         self._create_columns()
         self._widget.get_selection().connect(
             'changed', self._selection_changed, self.COLUMN_OBJ)
 
+    def _sort_feeds(self, model, a, b):
+        """
+        From the gtk.TreeSortable.set_sort_func doc:
+
+        The comparison callback should return -1 if the iter1 row should come before
+        the iter2 row, 0 if the rows are equal, or 1 if the iter1 row should come
+        after the iter2 row.
+        """
+        retval = 0
+        feed_a = model.get_value(a, self.COLUMN_OBJ)
+        feed_b = model.get_value(b, self.COLUMN_OBJ)
+
+        if feed_a in self._model.pseudo_categories:
+            retval = -1
+        elif feed_b in self._model.pseudo_categories:
+            retval = 1
+        elif feed_a is not None and feed_b is not None:
+            retval = locale.strcoll(feed_a.title, feed_b.title)
+        elif feed_a is not None: retval = -1
+        elif feed_b is not None: retval = 1
+        return retval
+
     def _model_set(self):
-        self._model.signal_connect(straw.FeedCategoryAddedSignal,
+        self._model.signal_connect(Event.FeedCategoryAddedSignal,
                                    self._category_added)
-        self._model.signal_connect(straw.FeedCategoryRemovedSignal,
+        self._model.signal_connect(Event.FeedCategoryRemovedSignal,
                                    self._category_removed)
-        self._model.signal_connect(straw.FeedCategoryChangedSignal,
+        self._model.signal_connect(Event.FeedCategoryChangedSignal,
                                    self._category_changed)
-        self._model.signal_connect(straw.FeedCategoryListLoadedSignal,
+        self._model.signal_connect(Event.FeedCategoryListLoadedSignal,
                                    self._categories_loaded)
         self._populate_treemodel()
 
     def _create_columns(self):
         renderer = gtk.CellRendererText()
         renderer.connect('edited', self._cell_edited)
-        column = gtk.TreeViewColumn(
-            _('_Category'), renderer, text=self.COLUMN_TITLE,
-            weight=self.COLUMN_BOLD, editable=self.COLUMN_EDITABLE)
+        column = gtk.TreeViewColumn(_('_Category'), renderer,
+                                    text=self.COLUMN_TITLE,
+                                    weight=self.COLUMN_BOLD,
+                                    editable=self.COLUMN_EDITABLE)
         self._widget.append_column(column)
 
     def _populate_treemodel(self):
@@ -53,13 +102,13 @@ class CategoryListView(MVP.WidgetView):
             treemodel.set(iter, self.COLUMN_TITLE, category.title,
                           self.COLUMN_OBJ, category,
                           self.COLUMN_BOLD, pango.WEIGHT_BOLD,
-                          self.COLUMN_EDITABLE, gtk.FALSE)
+                          self.COLUMN_EDITABLE, False)
         for category in self._model.user_categories:
             iter = treemodel.append()
             treemodel.set(iter, self.COLUMN_TITLE, category.title,
                           self.COLUMN_OBJ, category,
                           self.COLUMN_BOLD, pango.WEIGHT_NORMAL,
-                          self.COLUMN_EDITABLE, gtk.TRUE)
+                          self.COLUMN_EDITABLE, True)
 
     def _iterate_model_conditionally(self, func):
         treemodel = self._widget.get_model()
@@ -74,15 +123,24 @@ class CategoryListView(MVP.WidgetView):
             if treemodel.get_value(iter, self.COLUMN_OBJ) is category:
                 column = self._widget.get_column(self.COLUMN_TITLE)
                 path = treemodel.get_path(iter)
-                self._widget.set_cursor(path, column, gtk.TRUE)
+                self._widget.set_cursor(path, column, True)
                 return True
         self._iterate_model_conditionally(edit_row)
 
     def _remove_category(self, category):
         def remove_row(treemodel, iter):
             if treemodel.get_value(iter, self.COLUMN_OBJ) is category:
-                treemodel.remove(iter)
+                column = self._widget.get_column(self.COLUMN_TITLE)
+                iterpath = treemodel.get_path(iter)
+                isIter = treemodel.remove(iter)
+                if not isIter:
+                    current = iterpath[:-1] + (iterpath[-1]-1,)
+                else:
+                    current = treemodel.get_path(iter)
+                self._widget.set_cursor(current, column)
                 return True
+            return False
+        self._widget.grab_focus()
         self._iterate_model_conditionally(remove_row)
 
     def _update_category_name(self, category):
@@ -121,11 +179,11 @@ class CategoryListView(MVP.WidgetView):
         category = treemodel.get_value(iter, self.COLUMN_OBJ)
         self._presenter.category_title_edited(category, text)
 
-class CategorySelectionChangedSignal(straw.BaseSignal):
+class CategorySelectionChangedSignal(Event.BaseSignal):
     def __init__(self, sender, category):
-        straw.BaseSignal.__init__(self, sender)
+        Event.BaseSignal.__init__(self, sender)
         self.category = category
-    
+
 class CategoryListPresenter(MVP.BasicPresenter):
     """Presenter object for the category tree view"""
     def _initialize(self):
@@ -140,7 +198,7 @@ class CategoryListPresenter(MVP.BasicPresenter):
         category.title = title
 
     def add_category(self):
-        cat = straw.FeedCategoryList.FeedCategory(_("Category name"))
+        cat = FeedCategoryList.FeedCategory(_("Category name"))
         self._model.add_category(cat)
 
     def remove_category(self):
@@ -153,7 +211,7 @@ class CategoryAddView(MVP.WidgetView):
         if self._presenter is not None:
             self._presenter.add_category()
 
-class AddCategorySignal(straw.BaseSignal):
+class AddCategorySignal(Event.BaseSignal):
     pass
 
 class CategoryAddPresenter(MVP.BasicPresenter):
@@ -170,22 +228,22 @@ class CategoryRemoveView(MVP.WidgetView):
         if self._presenter is not None:
             self._presenter.remove_category()
 
-class RemoveCategorySignal(straw.BaseSignal):
+class RemoveCategorySignal(Event.BaseSignal):
     pass
 
 class CategoryRemovePresenter(MVP.BasicPresenter):
-    """Presenter object for the add category button"""
+    """Presenter object for the remove category button"""
     def _initialize(self):
         self.initialize_slots(RemoveCategorySignal)
 
     def remove_category(self):
         self.emit_signal(RemoveCategorySignal(self))
-        
-class CategoriesPresenter(object, straw.SignalEmitter):
+
+class CategoriesPresenter(object, Event.SignalEmitter):
     """Presenter object that combines the category list presenter and
     category add button presenter"""
     def __init__(self, lister, adder, remover):
-        straw.SignalEmitter.__init__(self)
+        Event.SignalEmitter.__init__(self)
         self.initialize_slots(CategorySelectionChangedSignal)
 
         lister.signal_connect(CategorySelectionChangedSignal,
@@ -210,10 +268,10 @@ class FeedListView(MVP.WidgetView):
     COLUMN_TOGGLABLE = 3
 
     def _initialize(self):
-        treemodel = gtk.TreeModelSort(
-            gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
-                          gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN))
+        treemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
+                                  gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN)
         treemodel.set_sort_func(self.COLUMN_TITLE, self._sort_feeds)
+        treemodel.set_sort_column_id(self.COLUMN_TITLE, gtk.SORT_ASCENDING)
         self._widget.set_model(treemodel)
         self._widget.set_rules_hint(False)
         self._create_columns()
@@ -222,8 +280,9 @@ class FeedListView(MVP.WidgetView):
         self._updating_display = False
 
     def _model_set(self):
-        self._model.signal_connect(straw.FeedsChangedSignal,
+        self._model.signal_connect(Event.FeedsChangedSignal,
                                    self._model_updated)
+        fclist = FeedCategoryList.get_instance()
         self._populate_treemodel()
 
     def _create_columns(self):
@@ -239,15 +298,15 @@ class FeedListView(MVP.WidgetView):
                                     text=self.COLUMN_TITLE)
         column.set_sort_column_id(self.COLUMN_TITLE)
         self._widget.append_column(column)
-        
+
     def _populate_treemodel(self):
-        treemodel = self._widget.get_model().get_model()
+        treemodel = self._widget.get_model()
         treemodel.clear()
-        feedlist = straw.FeedList.get_instance()
+        feedlist = FeedList.get_instance()
         if self._category is None: return
         category_feeds = self._category.feeds
         enabletoggle = not isinstance(
-            self._category, straw.FeedCategoryList.PseudoCategory)
+            self._category, FeedCategoryList.PseudoCategory)
         for feed in feedlist:
             iter = treemodel.append()
             treemodel.set(iter, self.COLUMN_TITLE, feed.title,
@@ -259,11 +318,12 @@ class FeedListView(MVP.WidgetView):
         feed_a = model.get_value(a, self.COLUMN_OBJ)
         feed_b = model.get_value(b, self.COLUMN_OBJ)
 
-        if feed_a is not None and feed_b is not None:
+        if feed_a and feed_b :
             return locale.strcoll(feed_a.title.lower(), feed_b.title.lower())
-        elif feed_a is not None: return 1
-        elif feed_b is not None: return -1
-        else: return 0
+        elif feed_a is not None: return -1
+        elif feed_b is not None: return 1
+        else:
+            return 0
 
     def _model_updated(self, signal):
         self._populate_treemodel()
@@ -276,10 +336,10 @@ class FeedListView(MVP.WidgetView):
     def _cell_toggled(self, cell, path_string):
         if self._updating_display:
             return
-        treemodel = self._widget.get_model().get_model()
-        iter = treemodel.get_iter_from_string(path_string)
-        if not iter: return
-        feed = treemodel.get_value(iter, self.COLUMN_OBJ)
+        treemodel = self._widget.get_model()
+        treeiter = treemodel.get_iter_from_string(path_string)
+        if not treeiter: return
+        feed = treemodel.get_value(treeiter, self.COLUMN_OBJ)
         self._presenter.feed_toggled(feed)
         self._populate_treemodel()
 
@@ -308,7 +368,7 @@ class CategoriesTab:
         self._selcategory = None
 
         clpresenter = CategoryListPresenter(
-            model = straw.FeedCategoryList.get_instance(),
+            model = FeedCategoryList.get_instance(),
             view = CategoryListView(xml.get_widget('category_treeview')))
         capresenter = CategoryAddPresenter(
             view = CategoryAddView(xml.get_widget('category_add_button')))
@@ -321,7 +381,7 @@ class CategoriesTab:
         self._categories_presenter.signal_connect(
             CategorySelectionChangedSignal, self._category_changed)
 
-        feedmodel = straw.FeedList.get_instance()
+        feedmodel = FeedList.get_instance()
         self._feeds_presenter = FeedListPresenter()
         self._feeds_presenter.model = feedmodel
         self._feeds_presenter.view = FeedListView(
@@ -360,7 +420,7 @@ class CategoriesTab:
         self._category_source_monitors.disconnect(oldcat)
         
         subscription_set = newcat.subscription is not None
-        is_pseudo = isinstance(newcat, straw.FeedCategoryList.PseudoCategory)
+        is_pseudo = isinstance(newcat, FeedCategoryList.PseudoCategory)
         default_refresh = (
             (not subscription_set) or
             (newcat.subscription.frequency ==
@@ -378,7 +438,7 @@ class CategoriesTab:
             w.set_sensitive(not is_pseudo)
         if default_refresh:
             self._source_frequency_spin.set_value(
-                straw.Config.get_instance().poll_frequency / 60)
+                Config.get_instance().poll_frequency / 60)
         else:
             self._source_frequency_spin.set_value(
                 (subscription_set and newcat.subscription.frequency / 60) or 0)
@@ -399,13 +459,13 @@ class CategoriesTab:
         def create_sub_ensurer(saver):
             def ensure_and_save(value, widget):
                 if newcat.subscription is None:
-                    newcat.subscription = straw.FeedCategoryList.OPMLCategorySubscription()
+                    newcat.subscription = FeedCategoryList.OPMLCategorySubscription()
                 saver(value, widget)
             return ensure_and_save
         
         if not self._category_source_monitors.has_key(newcat):
             monitors = dict(
-                [(key, straw.ValueMonitor.create_for_gtkentry(
+                [(key, ValueMonitor.create_for_gtkentry(
                 v, 5000, create_sub_ensurer(f)))
                 for key, v, f in (("location", self._source_field,
                                    save_location),
@@ -414,7 +474,7 @@ class CategoriesTab:
                                   ("password", self._source_password_field,
                                    save_password))])
             monitors.update(
-                {"frequency": straw.ValueMonitor.create_for_gtkspin(
+                {"frequency": ValueMonitor.create_for_gtkspin(
                 self._source_frequency_spin, 5000,
                 create_sub_ensurer(save_frequency))})
             self._category_source_monitors[newcat] = monitors
@@ -428,11 +488,10 @@ class CategoriesTab:
             self._selcategory, category)
         self._selcategory = category
 
-        if isinstance(category, straw.FeedCategoryList.PseudoCategory):
+        if isinstance(category, FeedCategoryList.PseudoCategory):
             self._delete_button.set_sensitive(False)
         else:
             self._delete_button.set_sensitive(True)
-
         self._feeds_presenter.category_selected(category)
         self._switching_categories = False
         return
@@ -447,9 +506,9 @@ class CategoriesTab:
     def _on_category_refresh_default_check_toggled(self, widget):
         if self._switching_categories:
             return
-        default_frequency = straw.Config.get_instance().poll_frequency / 60
+        default_frequency = Config.get_instance().poll_frequency / 60
         if self._selcategory.subscription is None:
-            self._selcategory.subscription = straw.FeedCategoryList.OPMLCategorySubscription()
+            self._selcategory.subscription = FeedCategoryList.OPMLCategorySubscription()
         if widget.get_active():
             self._source_frequency_spin.set_value(default_frequency)
             self._source_frequency_spin.set_sensitive(False)
@@ -466,16 +525,26 @@ class CategoriesTab:
 class PreferencesDialog:
     SEC_PER_MINUTE = 60
 
-    def __init__(self, xml):
-        config = straw.Config.get_instance()
+    def __init__(self, xml, parent):
+        config = Config.get_instance()
         self._window = xml.get_widget('preferences_dialog')
-        self._window.set_transient_for(straw.main_window.get_window())
+        self._window.set_transient_for(parent)
+        self._browser_override = xml.get_widget("prefs_browser_setting")
+        self._browser_entry = xml.get_widget("prefs_browser_setting_entry")
 
         # General
-        config = straw.Config.get_instance()
+        config = Config.get_instance()
         poll_frequency = int(config.poll_frequency/self.SEC_PER_MINUTE)
         items_stored = int(config.number_of_items_stored)
 
+        browser_cmd = str (config.browser_cmd)
+        if browser_cmd:
+            self._browser_override.set_active(True)
+            self._browser_entry.set_text (browser_cmd)
+            self._browser_entry.set_sensitive (True)
+        else:
+            self._browser_entry.set_text (_("Using desktop setting"))
+
         xml.get_widget('poll_frequency_spin').set_value(poll_frequency)
         xml.get_widget('number_of_items_spin').set_value(items_stored)
         xml.get_widget(['item_order_oldest',
@@ -498,25 +567,36 @@ class PreferencesDialog:
 
     def on_preferences_dialog_delete_event(self, *args):
         self.hide()
-        return gtk.TRUE
+        return True
 
     def on_preferences_close_button_clicked(self, button):
         self.hide()
         return
 
     def on_poll_frequency_spin_value_changed(self, spin):
-        straw.Config.get_instance().poll_frequency = int(spin.get_value() * self.SEC_PER_MINUTE)
+        Config.get_instance().poll_frequency = int(spin.get_value() * self.SEC_PER_MINUTE)
 
     def on_number_of_items_spin_value_changed(self, spin):
-        straw.Config.get_instance().number_of_items_stored = int(spin.get_value())
+        Config.get_instance().number_of_items_stored = int(spin.get_value())
 
     def on_item_order_newest_toggled(self, radio):
-        config = straw.Config.get_instance()
+        config = Config.get_instance()
         config.item_order = not config.item_order
 
     def on_item_order_oldest_toggled(self, radio):
         pass
 
+    def on_prefs_browser_setting_toggled(self, button):
+        active = button.get_active()
+        if not active:
+            config = Config.get_instance()
+            config.browser_cmd = ""
+        self._browser_entry.set_sensitive(active)
+
+    def on_prefs_browser_setting_entry_focus_out_event(self, entry, *args):
+        config = Config.get_instance()
+        config.browser_cmd = entry.get_text()
+
 class CategorySourceMonitors(WeakKeyDictionary):
     def disconnect(self, category):
         if self.has_key(category):
