@@ -1,6 +1,25 @@
+""" ValueMonitor.py
+
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """
+Straw is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+Straw is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
+
 import pygtk
 pygtk.require('2.0')
-import gtk
+import gobject
 import error
 
 class ValueMonitor(object):
@@ -20,7 +39,7 @@ class ValueMonitor(object):
         if self._connected:
             if value != self._value:
                 self._value_changed = True
-                gtk.timeout_add(self._timeout, self._check_value)
+                gobject.timeout_add(self._timeout, self._check_value)
             self._value = value
 
     value = property(get_value, set_value)
@@ -36,10 +55,7 @@ class ValueMonitor(object):
         self._check_value()
 
     def _check_value(self):
-        if (self._connected and
-            self._value_changed and
-            self.value != self._previous_value):
-            
+        if (self._connected and self._value_changed and self.value != self._previous_value):
             self._notify(self.value, self._data)
             self._previous_value = self.value
 
