@@ -91,8 +91,6 @@ def format_date(date, format=None, encoding=None):
         format = get_date_format()
     if encoding is None:
         encoding = get_locale_encoding()
-    if encoding.find('.') > 0:
-        encoding = encoding.split('.')[1] # get the encoding
     timestr = time.strftime(format, time.localtime(calendar.timegm(date)))
     return unicode(timestr, encoding)
 
@@ -168,13 +166,19 @@ def listdiff(l1, l2, test=None):
 import string
 import gnomevfs
 import Config
+import dialogs
+
 def url_show(url):
     config = Config.get_instance()
     if config.browser_cmd:
-        cmdbin, args = string.splitfields(str(config.browser_cmd), maxsplit=1)
-        link = args % url
-        pid = subprocess.Popen([cmdbin, link]).pid
-        return pid
+        try:
+            cmdbin, args = string.splitfields(str(config.browser_cmd), maxsplit=1)
+            link = args % url
+            pid = subprocess.Popen([cmdbin, link]).pid
+            return pid
+        except ValueError, ve:
+            dialogs.report_error(_("An error occurred while trying to open link"),
+                                 _("There was a problem opening '%s'\n\nError thrown is '%s'") % (url,str(ve)))
     else:
         return gnomevfs.url_show(url)
 
